@@ -1,9 +1,8 @@
-from yowsup.stacks import YowStack, YowStackBuilder
+from yowsup.stacks import  YowStackBuilder
 from .layer import YowsupCliLayer
 from yowsup.layers.auth import AuthError
 from yowsup.layers import YowLayerEvent
-from yowsup import env
-from yowsup.env import S40YowsupEnv
+from yowsup.layers.auth import YowAuthenticationProtocolLayer
 import sys
 
 class YowsupCliStack(object):
@@ -15,6 +14,7 @@ class YowsupCliStack(object):
             .push(YowsupCliLayer)\
             .build()
 
+        # self.stack.setCredentials(credentials)
         self.stack.setCredentials(credentials)
 
     def start(self):
@@ -27,4 +27,4 @@ class YowsupCliStack(object):
             print("Auth Error, reason %s" % e)
         except KeyboardInterrupt:
             print("\nYowsdown")
-            sys.exit(0)
\ No newline at end of file
+            sys.exit(0)
