@@ -11,22 +11,24 @@ class ProtocolEntityCallback(object):
         self.entityType = entityType
 
     def __call__(self, fn):
-        fn.callback = self.entityType
+        fn.entity_callback = self.entityType
         return fn
+    
 
 class YowInterfaceLayer(YowLayer):
 
     def __init__(self):
         super(YowInterfaceLayer, self).__init__()
-        self.callbacks = {}
+        self.entity_callbacks = {}
         self.iqRegistry = {}
         # self.receiptsRegistry = {}
         members = inspect.getmembers(self, predicate=inspect.ismethod)
         for m in members:
-            if hasattr(m[1], "callback"):
+            if hasattr(m[1], "entity_callback"):
                 fname = m[0]
                 fn = m[1]
-                self.callbacks[fn.callback] = getattr(self, fname)
+                self.entity_callbacks[fn.entity_callback] = getattr(self, fname)
+            
 
     def _sendIq(self, iqEntity, onSuccess = None, onError = None):
         assert iqEntity.getTag() == "iq", "Expected *IqProtocolEntity in _sendIq, got %s" % iqEntity.getTag()
@@ -94,11 +96,10 @@ class YowInterfaceLayer(YowLayer):
     def receive(self, entity):
         if not self.processIqRegistry(entity):
             entityType = entity.getTag()
-            if entityType in self.callbacks:
-                self.callbacks[entityType](entity)
+            if entityType in self.entity_callbacks:
+                self.entity_callbacks[entityType](entity)
             else:
                 self.toUpper(entity)
-
-
+ 
     def __str__(self):
         return "Interface Layer"
